/**
 * VueEventing (@rocketbase/vue-eventing v0.0.0)
 * Vue 2.x event support for Vue 3.x+
 * https://github.com/rocketbase-io/vue-eventing#readme
 * (c) 2020 Rocketbase Team <team@rocketbase.io>
 * @license MIT
 */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const config = {
    instanceMethods: false,
    emitIntegration: false,
};

function bus(handlers) {
    const listeners = {};
    /* eslint-disable-next-line */
    if (handlers)
        for (const [event, handler] of Object.entries(handlers))
            listeners[event] = typeof handler === "function" ? [handler] : handler;
    return {
        on(event, handler) {
            var _a;
            ((_a = listeners[event]) !== null && _a !== void 0 ? _a : (listeners[event] = [])).push(handler);
        },
        off(event, handler) {
            const handlers = listeners[event];
            if (handlers && handlers.includes(handler))
                handlers.splice(handlers.indexOf(handler), 1);
        },
        emit(event, ...args) {
            const handlers = listeners[event];
            if (!handlers)
                return;
            for (const handler of handlers) {
                handler(...args);
            }
        },
        reset(event) {
            delete listeners[event];
        },
    };
}

function installEmitIntegration(instance) {
    instance.$emit = wrapEmitter(instance.$emit);
}
function wrapEmitter(emitter) {
    return function (event, ...args) {
        const result = emitter === null || emitter === void 0 ? void 0 : emitter.call(this, event, ...args);
        this.$eventbus.emit(event, ...args);
        return result !== null && result !== void 0 ? result : this;
    };
}

function installInstanceMethods(cxt) {
    cxt.$on = instanceMethodOn;
    cxt.$off = instanceMethodOff;
    cxt.$once = instanceMethodOnce;
}
function instanceMethodOn(events, handler) {
    var _a;
    const { on } = (_a = this.$eventbus) !== null && _a !== void 0 ? _a : (this.$eventbus = bus());
    for (const event of typeof events === "string" ? [events] : events)
        on(event, handler);
    return this;
}
function instanceMethodOff(events, handler) {
    var _a;
    const { off } = (_a = this.$eventbus) !== null && _a !== void 0 ? _a : (this.$eventbus = bus());
    for (const event of typeof events === "string" ? [events] : events)
        off(event, handler);
    return this;
}
function instanceMethodOnce(events, handler) {
    var _a;
    const { on, off } = (_a = this.$eventbus) !== null && _a !== void 0 ? _a : (this.$eventbus = bus());
    for (const event of typeof events === "string" ? [events] : events) {
        on(event, function wrapper(...args) {
            off(event, wrapper);
            handler.call(this, ...args);
        });
    }
    return this;
}

function MixEventing(base) {
    return class extends (base !== null && base !== void 0 ? base : Object) {
        constructor(...args) {
            super(...args);
            this.$eventbus = bus();
            if (config.instanceMethods)
                installInstanceMethods(this);
            if (config.emitIntegration)
                installEmitIntegration(this);
        }
    };
}

function Eventing(cls) {
    const proto = cls.prototype;
    const beforeCreate = proto.beforeCreate;
    proto.beforeCreate = function (...args) {
        const cxt = this;
        const eventbus = (cxt.$eventbus = bus());
        if (config.emitIntegration)
            installEmitIntegration(cxt);
        eventbus.emit("hook:beforeCreate");
        return beforeCreate === null || beforeCreate === void 0 ? void 0 : beforeCreate.call(this, ...args);
    };
    if (config.instanceMethods)
        installInstanceMethods(proto);
    return cls;
}

function VueEventing({ instanceMethods, emitIntegration } = {}) {
    if (instanceMethods)
        config.instanceMethods = true;
    if (emitIntegration)
        config.emitIntegration = true;
}

exports.Eventing = Eventing;
exports.MixEventing = MixEventing;
exports.VueEventing = VueEventing;
exports.bus = bus;
//# sourceMappingURL=vue-eventing.cjs.js.map
